# Copyright (c) 2019 Forschungszentrum Juelich GmbH.
# This code is licensed under MIT license (see the LICENSE file for details).
# This code is derived from Tensorflow tutorials, which is licensed under the Apache License,
# Version 2.0 (see the NOTICE file for details).

"""
    This program is an adaptation of a previously available code sample
    at https://www.tensorflow.org/tutorials/. The program creates and trains a
    shallow ANN for handwritten digit classification using the MNIST dataset.

    To run this sample use the following command on your
    workstation/laptop:

        python -u mnist.py

    The code has been tested with Python 3.8.7 and tensorflow 2.3.1

    Note: This code will NOT work on the supercomputers.

"""

import tensorflow as tf


# Reference to the MNIST data object
mnist = tf.keras.datasets.mnist

# Load the MNIST dataset, split between train and test sets
(x_train, y_train), (x_test, y_test) = mnist.load_data()

# Normalize input samples
x_train, x_test = x_train / 255.0, x_test / 255.0

# Define the model, i.e., the network
model = tf.keras.models.Sequential([
    tf.keras.layers.Flatten(),
    tf.keras.layers.Dense(512, activation=tf.nn.relu),
    tf.keras.layers.Dense(10, activation=tf.nn.softmax)
])

# Optimizer
optimizer = tf.keras.optimizers.Adam()

# No. of epochs
epochs = 4

# Compile the model
model.compile(
    optimizer=optimizer,
    loss='sparse_categorical_crossentropy',
    metrics=['accuracy']
)

# Train the model using the training set
model.fit(
    x=x_train,
    y=y_train,
    batch_size=32,
    epochs=epochs,
    verbose=1
)

# Test the model using the test set
score = model.evaluate(x=x_test, y=y_test, verbose=0)
print(f'Test loss:  {score[0]}')
print(f'Test accuracy: {score[1]}')
